/********************************
 *
 * NAME:
 * ----
 * randbDoubleReceiver/main.c
 *
 * AUTHORS:
 * -------
 * Alvaro Gutierrez -- aguti@etsit.upm.es
 * Alexandre Campo  -- acampo@ulb.ac.be
 *
 * REVISION HISTORY:
 *-----------------
 * 12/12/2008
 *
 * ADDITIONAL NOTES:
 * ----------------
 * A test for the e-randb board.
 * The e-puck reads and send data 
 * to the E-RANDB 
 */


#include <p30f6014a.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <btcom.h>
#include <e_randb.h>

#include <e_uart_char.h>
#include <e_init_port.h>

int main() {
	
	/*system initialization */
	e_init_port();
	/* Init UART1 for bluetooth */
	e_init_uart1();
	/* Init E-RANDB board */
	e_init_randb();

	/* Wait for a command comming from bluetooth IRCOMTEST on pc directory*/
	btcomWaitForCommand('s');
	
	/* Range is tunable by software. 
	 * 0 -> Full Range (1m. approx depending on light conditions )
	 * 255 --> No Range (0cm. approx, depending on light conditions */
	e_randb_set_range(150);

	/* At some point we tought that the board could just take
	 * data en leave the calculations for the robot.
	 * At the moment, it is better to allow the board to do the calculations */
	e_randb_set_calculation(ON_BOARD);

	/* Store light conditions to use them as offset for the calculation 
	 * of the range and bearing */
	e_randb_store_light_conditions();

	/* A variable to hold a delay */
	
	/* Print on the bluetooth */
	char tmp2[50];
	sprintf(tmp2,"-- EMITTER/RECEIVER --\n");
	btcomSendString(tmp2);
	
	/* The counter for sending */
	unsigned int data = 0;

	long int delay;
	
	while(1){

		/* If something received */
		if(e_randb_get_if_received() != 0)
		{
			/* Get data */
			unsigned int data = e_randb_get_data();
			/* Get bearing */
			double bearing = e_randb_get_bearing();
			/*Get range */
			unsigned int range = e_randb_get_range();

			/* Print data received on the bluetooth */
			sprintf(tmp2,"%u %2f %u", data, bearing, range);
			btcomSendString(tmp2);
			btcomSendString("\n");
		
			for ( delay = 0 ; delay < 10000; delay++){
				__asm__("nop");
			}
		}
		
		/* Send data to the board */
		e_randb_send_all_data( 0xAA );

		/* A small delay */

		for ( delay = 0 ; delay < 10000; delay++){
			__asm__("nop");
		}
	}
	return 0;
}
